local *

utils =
  function:
    id: (x) ->
      x

    apply: (fn, a) ->
      (...) ->
        fn a, ...

  string:
    join: (s, t) ->
      table.concat t, s

    split: (sep, str) ->
      o = {}
      string.gsub str,
        (string.format "([^%s]+)", sep),
        utils.function.apply table.insert, o
      o

  table:
    list:
      map: (fn, t) ->
        [ fn v, i for i, v in ipairs t ]

      filter: (fn, t) ->
        [ v for i, v in ipairs t when fn v, i ]

      reduce: (fn, t, a) ->
        a = fn a, v, i for i, v in ipairs t
        a

      each: (fn, t) ->
        for i, v in ipairs t
          fn v, i

      index_of: (v, t) ->
        for i, x in ipairs t
          return i if x == v
        return false

      contains: (v, t) ->
        for i, x in ipairs t
          return true if v == x
        false

       -- lodash
      chunk: (s, t) ->
        o = {}
        c = {}
        for i, v in ipairs t
          if s == table.getn c
            table.insert o, c
            c = {}
          table.insert c, v
        o

      concat: (t, ...) ->
        utils.table.list.reduce (a, v) ->
          if (type v) == "table"
            utils.table.list.each(((x) -> table.insert a, x), v)
          else
            table.insert a, v
          a,
          {...},
          utils.table.list.map utils.function.id, t

    map:
      get: (k, t) ->
        parts = utils.string.split ".", k

        if #parts > 1
           -- TODO: Use head(t) and tail(t)
          first = parts[1]
          table.remove parts, 1
          rest = utils.string.join ".", parts

          utils.table.map.get rest, t[first]
        else
          t[k]

      set: (v, k, t) ->
        parts = utils.string.split ".", k

        if #parts > 1
           -- TODO: Use head(t) and tail(t)
          first = parts[1]
          table.remove parts, 1
          rest = utils.string.join ".", parts

          utils.table.map.set v, rest, t[first]
        else
          t[k] = v

      map: (fn, t) ->
        { k, fn v, k for k, v in pairs t }

      filter: (fn, t) ->
        { k, v for k, v in pairs t when fn v, k }

      reduce: (fn, t, a) ->
        a = fn a, v, k for k, v in pairs t
        a

      each: (fn, t) ->
        fn v, k for k, v in pairs t

      clone: (t) ->
        { k, v for k, v in pairs t }

      clone_deep: (t) ->
        { k, (((type v) ~= "table") and v or utils.table.map.clone_deep v) for k, v in pairs t }

      reverse: (t) ->
        { v, k for k, v in pairs t }

      merge: (...) ->
        utils.table.list.reduce ((a, x) ->
          utils.table.map.each ((v, k) ->
            a[k] = v
          ), x
          a
        ), {...}, {}

      keys: (t) ->
        [ k for k, v in pairs t ]

      values: (t) ->
        [ v for k, v in pairs t ]

  db:
    get_db_option: (k, t) ->
      parts = utils.string.split ".", k
      if #parts <= 1
        t[k]
      else
        first = parts[1]
        table.remove parts, 1
        rest = utils.string.join ".", parts
        utils.db.get_db_option rest, t[first].args

    set_db_option: (v, k, t) ->
      parts = utils.string.split ".", k
      if #parts <= 1
        t[k] = v
      else
        first = parts[1]
        table.remove parts, 1
        rest = utils.string.join ".", parts
        utils.db.set_db_option v, rest, t[first].args

idTweaksUtils = utils

_G.idTweaksUtils = utils

export idTweaksUtils
